<!DOCTYPE html>
<html lang="en">
<head>
  <meta charset="utf-8">
  <meta http-equiv="X-UA-Compatible" content="IE=edge">
  <meta name="viewport" content="width=device-width, initial-scale=1">
  {% if title %}
  <title>{{ title }}</title>
  {% else %}
  <title>Annif</title>
  {% endif %}
  <!-- Latest compiled and minified CSS -->
  <link rel="stylesheet" href="/static/css/bootstrap.min.css">
  <link rel="stylesheet" href="/static/css/style.css">
  <script src="/static/js/jquery-3.3.1.min.js"></script>
  <script src="/static/js/annif.js"></script>
  {% block head %}{% endblock %}
</head>
<body>

{% block content %}{% endblock %}

<script src="/static/js/bootstrap.min.js"></script>

</body>
</html>
